$Env:CONDA_EXE = "/opt/concourse/worker/volumes/live/8c917ec9-33fc-4310-458d-ebc5fd1453d7/volume/conda_1618432911252/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold/bin/conda"
$Env:_CE_M = ""
$Env:_CE_CONDA = ""
$Env:_CONDA_ROOT = "/opt/concourse/worker/volumes/live/8c917ec9-33fc-4310-458d-ebc5fd1453d7/volume/conda_1618432911252/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold"
$Env:_CONDA_EXE = "/opt/concourse/worker/volumes/live/8c917ec9-33fc-4310-458d-ebc5fd1453d7/volume/conda_1618432911252/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold/bin/conda"

Import-Module "$Env:_CONDA_ROOT\shell\condabin\Conda.psm1"
Add-CondaEnvironmentToPrompt